package liviu.tudor.catalog;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileFilter;
import java.util.Arrays;
import java.util.Comparator;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Panel which holds the GUI controls. This panel is placed in a JFrame when the
 * application starts, however this leaves the option of having it placed in any
 * other container (e.g. applet?)
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class CatalogUI extends JPanel {
    private static final long                  serialVersionUID = 1L;

    private static Logger                      LOGGER           = Logger.getLogger(CatalogUI.class.getName());

    private String                             directory;

    private JTextField                         fldDirectory;
    private javax.swing.filechooser.FileFilter chooseDirFilter;
    private JFileChooser                       fChooser;

    private JPanel                             pnlPicture;

    private JPanel                             pnlSubdirs;

    private JTextField                         fldFile;

    private FileFilter                         filterDirs;
    private File                               subdirs[];
    private Comparator<File>                   subdirsComparator;

    private FileFilter                         filterFiles;
    private File                               images[];
    private int                                currImgIdx;
    private Image                              currImage;

    /**
     * The one and only ... Constructor :-)
     *
     * @param directory
     *            Directory to start searching for pictures in
     */
    public CatalogUI(String directory) {
        setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

        /*
         * Directory controls
         */
        JPanel dirCtrls = new JPanel();
        dirCtrls.setLayout(new FlowLayout());
        fldDirectory = new JTextField(50);
        fldDirectory.setEditable(false);
        if (directory != null) fldDirectory.setText(directory);
        dirCtrls.add(fldDirectory);
        JButton btnChangeDir = new JButton("...");
        dirCtrls.add(btnChangeDir);
        btnChangeDir.addActionListener(new ChangeDirListener());
        JButton btnRefresh = new JButton("Refresh");
        dirCtrls.add(btnRefresh);
        btnRefresh.addActionListener(new RefreshListener());
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridheight = 1;
        gbc.gridwidth = 11;
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1;
        gbc.weighty = 0.09;
        add(dirCtrls, gbc);

        /*
         * Next/Delete
         */
        JPanel pnlNextDel = new JPanel();
        pnlNextDel.setLayout(new GridLayout(2, 1));
        JButton btnDelCurrent = new JButton("Del");
        btnDelCurrent.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
                if ((images == null) || (images.length <= currImgIdx)) return;
                images[currImgIdx].delete();
                setCurrentPicture(currImgIdx + 1);
            }
        });
        pnlNextDel.add(btnDelCurrent);
        JButton btnNext = new JButton("Next");
        btnNext.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
                setCurrentPicture(currImgIdx + 1);
            }
        });
        pnlNextDel.add(btnNext);
        gbc.gridx = 10;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.gridheight = 2;
        gbc.weightx = 0.1;
        gbc.weighty = 0.18;
        add(pnlNextDel, gbc);

        /*
         * Current picture
         */
        fldFile = new JTextField(50);
        fldFile.setEditable(false);
        gbc.gridy = 11;
        gbc.gridx = 0;
        gbc.gridwidth = 11;
        gbc.gridheight = 1;
        gbc.weightx = 1;
        gbc.weighty = 0.09;
        add(fldFile, gbc);

        /*
         * Picture area
         */
        pnlPicture = new PicturePanel();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 10;
        gbc.gridheight = 10;
        gbc.weightx = 0.9;
        gbc.weighty = 0.9;
        Dimension d = new Dimension(640, 480);
        pnlPicture.setMinimumSize(d);
        pnlPicture.setPreferredSize(d);
        add(pnlPicture, gbc);

        /*
         * Directories
         */
        pnlSubdirs = new JPanel();
        gbc.gridx = 10;
        gbc.gridy = 3;
        gbc.gridwidth = 1;
        gbc.gridheight = 8;
        gbc.weightx = 0.1;
        gbc.weighty = 0.72;
        setDirectory(directory);
        add(pnlSubdirs, gbc);
    }

    protected void setDirectory(String directory) {
        this.directory = directory;
        if (this.directory == null) this.directory = "";
        fldDirectory.setText(this.directory);
        pnlSubdirs.removeAll();
        if ("".equals(this.directory)) return;
        File fDir = new File(directory);
        if (!fDir.exists() || !fDir.isDirectory()) return;
        if (filterDirs == null) filterDirs = new DirFilter();
        subdirs = fDir.listFiles(filterDirs);

        if ((subdirs != null) && (subdirs.length > 0)) {
            if (subdirsComparator == null) subdirsComparator = new SubdirsNameComparator();
            Arrays.sort(subdirs, subdirsComparator);
            pnlSubdirs.setLayout(new GridLayout(subdirs.length, 1));
            for (int i = 0; i < subdirs.length; i++) {
                JButton btn = new JButton(subdirs[i].getName());
                btn.addActionListener(new SubdirActionListener(subdirs[i]));
                pnlSubdirs.add(btn);
            }
        }

        if (filterFiles == null) filterFiles = new FileOnlyFilter();
        images = fDir.listFiles(filterFiles);
        Arrays.sort(images, subdirsComparator);
        setCurrentPicture(0);
        pnlSubdirs.repaint();

        Catalog.saveSetting(this.directory);
    }

    protected void setCurrentPicture(int idx) {
        if (images == null) return;
        if (idx >= images.length) // the end!
        {
            images = null;
            return;
        }
        if (!isImage(idx)) {
            setCurrentPicture(idx + 1);
            return;
        }
        currImgIdx = idx;
        fldFile.setText(images[currImgIdx].getAbsolutePath());
        currImage = Toolkit.getDefaultToolkit().createImage(images[currImgIdx].getAbsolutePath());
        pnlPicture.repaint();
    }

    protected boolean isImage(int idx) {
        String name = images[idx].getName().toLowerCase();
        if (name.endsWith(".jpg")) return true;
        if (name.endsWith(".jpeg")) return true;
        if (name.endsWith(".gif")) return true;
        if (name.endsWith(".png")) return true;

        return false;
    }

    private static final char[] CHARS = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=', '+', '_',
            'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'z', 'x',
            'c', 'v', 'b', 'n', 'm'  };

    protected String appendRandomToName(String fileName) {
        int posLastDot = fileName.lastIndexOf('.');
        String sfileName = "", suffix = "";
        int index = (int) (Math.random() * (CHARS.length + 1));
        if (index >= CHARS.length) index = 0;
        if (posLastDot > 0) {
            sfileName = fileName.substring(0, posLastDot);
            if (posLastDot < fileName.length()) suffix = fileName.substring(posLastDot + 1);
        } else
            sfileName = fileName;
        return sfileName + CHARS[index] + '.' + suffix;
    }

    class SubdirsNameComparator implements Comparator<File> {

        @Override
        public int compare(File dir1, File dir2) {
            if (dir1 == null) return -1;
            if (dir2 == null) return 1;
            String s1 = dir1.getName().toLowerCase();
            String s2 = dir2.getName().toLowerCase();
            return s1.compareTo(s2);
        }

    }

    class SubdirActionListener implements ActionListener {
        File subdir;

        SubdirActionListener(File subdir) {
            this.subdir = subdir;
        }

        @Override
        public void actionPerformed(ActionEvent event) {
            if ((images == null) || (currImgIdx >= images.length)) return;
            String fileName = subdir.getAbsolutePath() + File.separator + images[currImgIdx].getName();
            File dest = new File(fileName);
            LOGGER.fine("Trying to move " + images[currImgIdx].getAbsolutePath() + " to " + fileName);
            while (dest.exists()) {
                LOGGER.fine("Can't... trying appending to name...");
                fileName = appendRandomToName(fileName);
                dest = new File(fileName);
                LOGGER.fine("Trying to move " + images[currImgIdx].getAbsolutePath() + " to " + fileName);
            }
            images[currImgIdx].renameTo(dest);
            setCurrentPicture(currImgIdx + 1);
        }
    }

    class DirFilter implements FileFilter {
        @Override
        public boolean accept(File f) {
            if (!f.isDirectory()) return false;
            if (".".equals(f.getName()) || "..".equals(f.getName())) return false;
            return true;
        }
    }

    class AllDirFilters extends javax.swing.filechooser.FileFilter {
        @Override
        public boolean accept(File f) {
            if (!f.isDirectory()) return false;
            return true;
        }

        @Override
        public String getDescription() {
            return "All Directories";
        }
    }

    class FileOnlyFilter implements FileFilter {
        @Override
        public boolean accept(File f) {
            if (f.isDirectory()) return false;
            return true;
        }
    }

    class ChangeDirListener implements ActionListener {
        @Override
        public void actionPerformed(ActionEvent event) {
            if (fChooser == null) {
                fChooser = new JFileChooser();
                fChooser.setDialogTitle("Choose directory to catalog");
                fChooser.setDialogType(JFileChooser.OPEN_DIALOG);
                fChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                if (chooseDirFilter == null) chooseDirFilter = new AllDirFilters();
                fChooser.setFileFilter(chooseDirFilter);
            }
            fChooser.setCurrentDirectory(new File(directory));
            int retVal = fChooser.showOpenDialog(CatalogUI.this);
            if (retVal == JFileChooser.APPROVE_OPTION) setDirectory(fChooser.getSelectedFile().getAbsolutePath());
        }
    }

    class RefreshListener implements ActionListener {
        @Override
        public void actionPerformed(ActionEvent event) {
            setDirectory(directory);
        }
    }

    class PicturePanel extends JPanel {
        private static final long serialVersionUID = 1L;

        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            if (currImage == null) return;
            g.drawImage(currImage, 0, 0, getWidth(), getHeight(), this);
        }
    }
}
